@@ -21,13 +21,13 @@ Copyright (c) Intel Corporation (2009-2017).
 #ifndef OPENCL_CLANG_OPTIONS_H
 #define OPENCL_CLANG_OPTIONS_H
 
-#include "LLVMSPIRVOpts.h"
-#include "clang/Basic/OpenCLOptions.h"
+#include "llvm/Option/Arg.h"
 #include "llvm/ADT/ArrayRef.h"
 #include "llvm/ADT/StringRef.h"
-#include "llvm/Option/Arg.h"
 #include "llvm/Option/ArgList.h"
 #include "llvm/Option/Option.h"
+#include "clang/Basic/OpenCLOptions.h"
+#include "LLVMSPIRVOpts.h"
 
 #include <list>
 
@@ -146,7 +146,7 @@ private:
 class CompileOptionsParser {
 public:
   CompileOptionsParser(const char *pszOpenCLVersion)
-      : m_commonFilter(pszOpenCLVersion), m_emitSPIRV(false), m_optDisable(false), m_opaquePointers(true) {}
+      : m_commonFilter(pszOpenCLVersion), m_emitSPIRV(false), m_optDisable(false) {}
 
   //
   // Validates and prepares the effective options to pass to clang upon
@@ -186,7 +186,6 @@ public:
   }
 
   bool hasOptDisable() const { return m_optDisable; }
-  bool hasOpaquePointers() const { return m_opaquePointers; }
 
 private:
   OpenCLCompileOptTable m_optTbl;
@@ -198,7 +197,6 @@ private:
   bool m_hasSPIRVExt = false;
   SPIRV::TranslatorOpts::ExtensionsStatusMap m_SPIRVExtStatusMap = {};
   bool m_optDisable;
-  bool m_opaquePointers;
 };
 
 // Tokenize a string into tokens separated by any char in 'delims'.
