@@ -20,6 +20,7 @@ Copyright (c) Intel Corporation (2009-2017).
 #include "options.h"
 
 #include "clang/Driver/Options.h"
+#include "llvm/ADT/SmallSet.h"
 #include "llvm/ADT/StringExtras.h"
 #include "llvm/Option/Arg.h"
 #include "llvm/Option/ArgList.h"
@@ -42,7 +43,7 @@ Copyright (c) Intel Corporation (2009-2017).
 
 using namespace llvm::opt;
 
-static llvm::ManagedStatic<llvm::sys::SmartMutex<true> > compileOptionsMutex;
+extern llvm::ManagedStatic<llvm::sys::SmartMutex<true>> compileMutex;
 
 static constexpr OptTable::Info ClangOptionsInfoTable[] = {
 #define PREFIX(NAME, VALUE)
@@ -62,6 +63,70 @@ OpenCLCompileOptTable::OpenCLCompileOptTable()
 
 int EffectiveOptionsFilter::s_progID = 1;
 
+// This code was adopted from the SPIRV-LLVM-Translator repository.
+static int parseSPVExtOption(
+    llvm::StringRef SPVExt,
+    SPIRV::TranslatorOpts::ExtensionsStatusMap &ExtensionsStatus) {
+  // Map name -> id for known extensions
+  std::map<llvm::StringRef, SPIRV::ExtensionID> ExtensionNamesMap;
+#define _STRINGIFY(X) #X
+#define STRINGIFY(X) _STRINGIFY(X)
+#define EXT(X) ExtensionNamesMap[STRINGIFY(X)] = SPIRV::ExtensionID::X;
+#include "LLVMSPIRVExtensions.inc"
+#undef EXT
+#undef STRINGIFY
+#undef _STRINGIFY
+
+  // Set the initial state: assume that any known extension is disallowed.
+  std::optional<bool> DefaultVal;
+  for (const auto &It : ExtensionNamesMap)
+    ExtensionsStatus[It.second] = DefaultVal;
+
+  llvm::SmallVector<llvm::StringRef, 32> SPVExtList;
+  llvm::SplitString(SPVExt, SPVExtList, ",");
+
+  if (SPVExtList.empty()) {
+    llvm::errs() << "Invalid value of --spirv-ext, expected format is:\n"
+                 << "\t--spirv-ext=+EXT_NAME,-EXT_NAME\n";
+    return -1;
+  }
+
+  for (unsigned i = 0; i < SPVExtList.size(); ++i) {
+    llvm::StringRef ExtString = SPVExtList[i];
+    if (ExtString.empty() ||
+        ('+' != ExtString.front() && '-' != ExtString.front())) {
+      llvm::errs() << "Invalid value of --spirv-ext, expected format is:\n"
+                   << "\t--spirv-ext=+EXT_NAME,-EXT_NAME\n";
+      return -1;
+    }
+
+    auto ExtName = ExtString.substr(1);
+
+    if (ExtName.empty()) {
+      llvm::errs() << "Invalid value of --spirv-ext, expected format is:\n"
+                   << "\t--spirv-ext=+EXT_NAME,-EXT_NAME\n";
+      return -1;
+    }
+
+    bool ExtStatus = ('+' == ExtString.front());
+    if ("all" == ExtName) {
+      // Update status for all known extensions
+      for (const auto &It : ExtensionNamesMap)
+        ExtensionsStatus[It.second] = ExtStatus;
+    } else {
+      // Ignore unknown extensions, as targets may support non-standard SPIR-V
+      // extensions. Do not reject them; this approach is more tolerant.
+      const auto &It = ExtensionNamesMap.find(ExtName);
+      if (ExtensionNamesMap.end() == It)
+        continue;
+
+      ExtensionsStatus[It->second] = ExtStatus;
+    }
+  }
+
+  return 0;
+}
+
 ///
 // Options filter that validates the opencl used options
 //
@@ -70,6 +135,7 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
                                                    ArgsVector &effectiveArgs) {
   // Reset args
   int iCLStdSet = 0;
+  bool isCpp = false;
   bool fp64Enabled = false;
   std::string szTriple;
   std::string sourceName(llvm::Twine(s_progID++).str());
@@ -138,6 +204,17 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
       iCLStdSet = 300;
       effectiveArgs.push_back((*it)->getAsString(args));
       break;
+    case OPT_COMPILE_cl_std_CLCxx:
+    case OPT_COMPILE_cl_std_CLCxx1_0:
+      iCLStdSet = 200;
+      isCpp = true;
+      effectiveArgs.push_back((*it)->getAsString(args));
+      break;
+    case OPT_COMPILE_cl_std_CLCxx2021:
+      iCLStdSet = 300;
+      isCpp = true;
+      effectiveArgs.push_back((*it)->getAsString(args));
+      break;
     case OPT_COMPILE_triple:
       szTriple = (*it)->getValue();
       break;
@@ -219,7 +296,9 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
 
   // Specifying the option makes clang emit function body for functions
   // marked with inline keyword.
-  effectiveArgs.push_back("-fgnu89-inline");
+  if (!isCpp) {
+    effectiveArgs.push_back("-fgnu89-inline");
+  }
 
   // Do not support all extensions by default. Support for a particular
   // extension should be enabled by passing a '-cl-ext' option in pszOptionsEx.
@@ -259,7 +338,7 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
   std::map<std::string, bool> extMap;
   llvm::SmallVector<llvm::StringRef> extVec;
   llvm::SplitString(PCH_EXTENSION, extVec, ",");
-  for(auto ext : extVec)
+  for (auto &ext : extVec)
     extMap.insert({ext.str(), true});
 #else
   std::map<std::string, bool> extMap{
@@ -287,10 +366,12 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
 
   auto parseClExt = [&](const std::string &clExtStr) {
     llvm::StringRef clExtRef(clExtStr);
-    clExtRef.consume_front("-cl-ext=");
+    bool hasPrefix = clExtRef.consume_front("-cl-ext=");
+    assert(hasPrefix && "clExtRef doesn't start with \"-cl-ext\" prefix");
+    (void)hasPrefix;
     llvm::SmallVector<llvm::StringRef, 32> parsedExt;
     clExtRef.split(parsedExt, ',');
-    for (auto ext : parsedExt) {
+    for (auto &ext : parsedExt) {
       char sign = ext.front();
       bool enabled = sign != '-';
       llvm::StringRef extName = ext;
@@ -306,11 +387,30 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
         it->second = enabled;
     }
   };
+  llvm::SmallSet<llvm::StringRef, 32> parsedOclCFeatures;
   std::for_each(effectiveArgs.begin(), effectiveArgs.end(),
                 [&](const ArgsVector::value_type &a) {
                   if (a.find("-cl-ext=") == 0)
                     parseClExt(a);
+		  else if (a.find("-D__opencl_c_") == 0)
+		    parsedOclCFeatures.insert(a);
                 });
+
+  // "opencl-c-base.h" unconditionally enables a list of so-called "optional
+  // core" language features. We need to undef those that aren't explicitly
+  // defined within the compilation command (which would suggest that the
+  // target platform supports the corresponding feature).
+  const char* optionalCoreOclCFeaturesList[] = {
+      "__opencl_c_work_group_collective_functions",
+      "__opencl_c_atomic_order_seq_cst",
+      "__opencl_c_atomic_scope_device",
+      "__opencl_c_atomic_scope_all_devices",
+      "__opencl_c_read_write_images" };
+  for (std::string OclCFeature : optionalCoreOclCFeaturesList) {
+    if (!parsedOclCFeatures.contains(std::string("-D") + OclCFeature))
+      effectiveArgs.push_back(std::string("-D__undef_") + OclCFeature);
+  }
+
   // extension is enabled in PCH but disabled or not specifed in options =>
   // disable pch
   bool useModules =
@@ -361,8 +461,8 @@ std::string EffectiveOptionsFilter::processOptions(const OpenCLArgList &args,
   return sourceName;
 }
 
-void CompileOptionsParser::processOptions(const char *pszOptions,
-                                          const char *pszOptionsEx) {
+int CompileOptionsParser::processOptions(const char *pszOptions,
+                                         const char *pszOptionsEx) {
   // parse options
   unsigned missingArgIndex, missingArgCount;
   std::unique_ptr<OpenCLArgList> pArgs(
@@ -376,16 +476,27 @@ void CompileOptionsParser::processOptions(const char *pszOptions,
   for (ArgsVector::iterator it = m_effectiveArgs.begin(),
                             end = m_effectiveArgs.end();
        it != end; ++it) {
-    if (it->compare("-cl-opt-disable") == 0) {
+    llvm::StringRef arg(*it);
+    (void)arg.consume_front("-");
+    // support -- prefix as well.
+    (void)arg.consume_front("-");
+    if (arg == "cl-opt-disable") {
       m_optDisable = true;
-    }
-    else if (it->compare("-emit-spirv") == 0) {
+    } else if (arg == "emit-spirv") {
       m_effectiveArgsRaw.push_back("-emit-llvm-bc");
       m_emitSPIRV = true;
       continue;
+    } else if (arg.consume_front("spirv-ext=")) {
+      m_hasSPIRVExt = true;
+      // m_SPIRVExtStatusMap will be initialized and updated according to `arg`.
+      int ret = parseSPVExtOption(arg, m_SPIRVExtStatusMap);
+      if (0 != ret)
+        return ret;
+      continue;
     }
     m_effectiveArgsRaw.push_back(it->c_str());
   }
+  return 0;
 }
 
 bool CompileOptionsParser::checkOptions(const char *pszOptions,
@@ -439,7 +550,7 @@ extern "C" CC_DLL_EXPORT bool CheckCompileOptions(const char *pszOptions,
                                                   size_t uiUnknownOptionsSize) {
   // LLVM doesn't guarantee thread safety,
   // therefore we serialize execution of LLVM code.
-  llvm::sys::SmartScopedLock<true> compileOptionsGuard {*compileOptionsMutex};
+  llvm::sys::SmartScopedLock<true> compileOptionsGuard{*compileMutex};
 
   try {
     CompileOptionsParser optionsParser("200");
